/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 /*
    Author:
    Pawe (Googie) Salawa
    boogie@myslenice.one.pl

    ChangeLog:

        [v1.5]
            * Added /usr/include/kde to spec includes path.
            * Removed deprecated getActualStatus()
 
        [v1.4]
            * Added %d tago to descriptions. It's replaced by
              any description you set manualy.
  
        [v1.3]
            * Now amaroKs tab in Kadu configuration dialog
              is removed after unload module.

        [v1.2]
            * Provides 'descriptions' property and makes
              conflicts with other module which provides this.

        [v1.1]
            * Updated for Kadu 0.3.8-cvs (04.04.2004)

        [v1.0]
            * First public release.
*/
 
#include <qtimer.h>

#include <kapplication.h>

#include "debug.h"
#include "kadu.h"
#include "message_box.h"
#include "userlist.h"
#include "config_dialog.h"
#include "status.h"
#include "modules.h"

#include "amarok.h"

#define MODULE_AMAROK_VERSION 1.5

AmaroK* amarok;

extern "C" void amarok_info(ModuleInfo* i)
{
    i->description = QObject::tr("amaroK player support for Kadu");
    i->author = "Googie";
}

extern "C" int amarok_init()
{
    amarok = new AmaroK();
    return 0;
}

extern "C" void amarok_close()
{
    delete amarok;
}

AmaroK::AmaroK()
{
    client = kapp->dcopClient();
    client->attach();

    menu = new QPopupMenu();
    popups[0] = menu->insertItem(tr("Put formated string"), this, SLOT(putSongTitle(int)));
    popups[1] = menu->insertItem(tr("Put song title"), this, SLOT(putSongTitle(int)));

    timer = new QTimer();
    connect(timer, SIGNAL(timeout()), this, SLOT(checkTitle()));
    popups[2] = kadu->mainMenu()->insertItem(tr("Enable amaroK statuses"), this, SLOT(toggleTimer(int)), 0, -1, 0);

    connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));

    ConfigDialog::addTab("amaroK");
    ConfigDialog::addLineEdit("amaroK", "amaroK", QT_TRANSLATE_NOOP("@default", "amaroK message format for chat windows"), "chatString", "amaroK: %t [%c / %l]",
        QT_TRANSLATE_NOOP("@default", "%t - song title, %f - file name, %l - song length (MM:SS),<br>%c - current song position (MM:SS), %p - percents of played song"));
    ConfigDialog::addLineEdit("amaroK", "amaroK", QT_TRANSLATE_NOOP("@default", "amaroK status format"), "statusString", "-=[%t]=-",
        QT_TRANSLATE_NOOP("@default", "%t - song title, %f - file name, %l - song length (MM:SS),<br>%c - current song position (MM:SS), %p - percents of played song"));

    descriptionToUse = "";
    descriptionToSet = "";
    amarokSets = FALSE;
}

AmaroK::~AmaroK()
{
    delete client;
    delete menu;
    delete timer;
    kadu->mainMenu()->removeItem(popups[2]);

    disconnect(timer, SIGNAL(timeout()), this, SLOT(checkTitle()));
    disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));

    ConfigDialog::removeControl("amaroK", "amaroK message format for chat windows");
    ConfigDialog::removeControl("amaroK", "amaroK status format");
    ConfigDialog::removeTab("amaroK");
}

bool AmaroK::isPlaying()
{
    bool b;
    QByteArray replyData, data;
    QCString replyType;
    if (!client->call("amarok", "player", "isPlaying()", data, replyType, replyData))
        return false;

    QDataStream reply(replyData, IO_ReadOnly);
    reply >> b;
    return b;
}

QString AmaroK::getTitle()
{
    kdebugf();
    QByteArray replyData, data;
    QCString replyType;
    client->call("amarok", "player", "nowPlaying()", data, replyType, replyData);
    QDataStream reply(replyData, IO_ReadOnly);
    QString curTrack;
    reply >> curTrack;
    return curTrack.simplifyWhiteSpace();
}

int AmaroK::getLength()
{
    kdebugf();
    QByteArray replyData, data;
    QCString replyType;
    int t;
    client->call("amarok", "player", "trackTotalTime()", data, replyType, replyData);
    QDataStream reply(replyData, IO_ReadOnly);
    reply >> t;
    return t;
}

int AmaroK::getCurrentPos()
{
    kdebugf();
    QByteArray replyData, data;
    QCString replyType;
    int pos;
    client->call("amarok", "player", "trackCurrentTime()", data, replyType, replyData);
    QDataStream reply3(replyData, IO_ReadOnly);
    reply3 >> pos;
    return pos;
}

void AmaroK::toggleTimer(int id)
{
    kdebugf();
    if (!isPlaying())
    {
        MessageBox::msg(tr("amaroK isn't playing!"));
        return;
    }
    if (timer->isActive())
    {
        timer->stop();
        kadu->mainMenu()->changeItem(popups[2], tr("Enable amaroK statuses"));
    }
    else
    {
        kadu->mainMenu()->changeItem(popups[2], tr("Disable amaroK statuses"));
        timer->start(1000);
    }
}

void AmaroK::checkTitle()
{
    int status = getCurrentStatus(), act;
    switch (GG_S(status))
    {
        case GG_STATUS_AVAIL:
        case GG_STATUS_BUSY:
        case GG_STATUS_INVISIBLE:
            act = 1;
            break;
        case GG_STATUS_AVAIL_DESCR:
        case GG_STATUS_BUSY_DESCR:
        case GG_STATUS_INVISIBLE_DESCR:
            act = 2;
            break;
        default:
            act = 0;
    }
    if (act)
    {
        QString title = getTitle();
        if (title != currentTitle || act == 1)
        {
            currentTitle = title;
            if (act == 1)
                status += 2;

            own_description = parse(config_file.readEntry("amaroK", "statusString"));
            amarokSets = TRUE;
            kadu->setStatus(status);
            amarokSets = FALSE;
        }
    }
}

void AmaroK::chatCreated(const UinsList& ul)
{
    kdebugf();
    Chat* chat = chat_manager->findChatByUins(ul);
    
    QPushButton* amarok = new QPushButton(chat->buttontray);
    amarok->setPixmap(icons_manager.loadIcon("Unmute"));
    amarok->setPopup(menu);
    amarok->show();
    QToolTip::add(amarok, "amaroK");
    
}

void AmaroK::putSongTitle(int ident)
{
    kdebugf();
    if (!isPlaying())
    {
        MessageBox::msg(tr("amaroK isn't playing!"));
        return;
    }
    Chat* chat = getCurrentChat();
    QString title;
    if (popups[0] == ident)
        title = parse(config_file.readEntry("amaroK", "chatString"));

    if (popups[1] == ident)
        title = getTitle();

    int x, y;

    HtmlDocument doc, doc2, doc3;
    chat->edit->getCursorPosition(&y, &x);
    chat->edit->insertAt(title, y, x);
    doc.parseHtml(chat->edit->text());
    for ( int i = 0; i < doc.countElements(); i++ )
    {
        if (i == 7)
        {
            doc2.parseHtml(doc.elementText(i));
            for ( int j = 0; j < doc2.countElements(); j++ )
            {
                if (doc2.isTagElement(j))
                    doc3.addTag(doc2.elementText(j));
                else
                    doc3.addText(doc2.elementText(j));
            }
        } else {
            if (doc.isTagElement(i))
                doc3.addTag(doc.elementText(i));
            else
                doc3.addText(doc.elementText(i));
        }
    }
    chat->edit->setText(doc3.generateHtml());
    chat->edit->moveCursor(QTextEdit::MoveEnd, FALSE);
}

QString AmaroK::parse(QString str)
{
    kdebugf();
    uint sl = str.length();
    QString r;

    for ( uint i = 0; i < sl; i++ )
    {
        while (str[i] != '%' && i < sl)
        {
            r += str[i];
            i++;
        }

        if (str[i] == '%')
        {
            i++;
            switch(str[i].latin1())
            {
                case 't':
                {
                    r += getTitle();
                    break;
                }
                case 'l':
                {
                    char ms[10];
                    int lgt = getLength() / 1000, m, s;
                    m = lgt / 60;
                    s = lgt % 60;
                    if (s < 10)
                        sprintf(ms, "%d:0%d", m, s);
                    else
                        sprintf(ms, "%d:%d", m, s);

                    r += ms;
                    break;
                }
                case 'c':
                {
                    char ms[10];
                    int lgt = getCurrentPos(), m, s;
                    m = lgt / 60;
                    s = lgt % 60;
                    if (s < 10)
                        sprintf(ms, "%d:0%d", m, s);
                    else
                        sprintf(ms, "%d:%d", m, s);

                    r += ms;
                    break;
                }
                case 'p':
                {
                    char tmp[10];
                    int perc = getCurrentPos() / (getLength()/1000);
                    sprintf(tmp, "%d%%", perc);
                    r += tmp;
                    break;
                }
                case 'd':
                {
                    r += descriptionToUse;
                    break;
                }
                default:
                    r += str[i];
            }
        }
    }
    return r;
}

Chat* AmaroK::getCurrentChat()
{
    kdebugf();
    ChatList cs = chat_manager->chats();
    uint i;
    for ( i = 0; i < cs.count(); i++ )
    {
        if (cs[i]->isActiveWindow())
        {
            break;
        }
    }
    if (i == cs.count())
        return 0;

    return cs[i];
}

void AmaroK::checkStatusDesc(int status, QString desc, bool &stop)
{
    QString str = config_file.readEntry("amaroK", "statusString");
    if (str.find("%d") > -1 && desc != "" && descriptionToSet != desc && !amarokSets)
    {
        stop = TRUE;
        descriptionToUse = desc;
        own_description = parse(str);
        descriptionToSet = own_description;
        kadu->setStatus(status);
    }
}
